################################################################
##
## MySQL Database Class
from __future__ import division
import rankme.pymysql as pymysql
from rankme import rankme
import es, playerlib
import re

class MySQLDataBase:
    ''' The ranking database, handles all off the data '''
    def __init__(self):
        ''' Fires when the script is initialized '''
        # Create a connection to our database
        self.connection = pymysql.connect(host=rankme.settings.hostname, user=rankme.settings.username, passwd=rankme.settings.password)
        self.connection.charset = "utf8"
        # Establish a cursor for data handling
        self.cursor = self.connection.cursor()
        try:
            self.cursor.execute("CREATE DATABASE IF NOT EXISTS %s" % rankme.settings.db)
        except:
            pass
        
        try:
            self.cursor.execute("USE %s" % rankme.settings.db) 
        except:
            raise ValueError("[rankME] Cannot connect to database %s and do not have permission to create either" % rankme.settings.db)
        
        self.cursor.execute('SET NAMES utf8')
        
        self.cursor.execute("""\
        CREATE TABLE IF NOT EXISTS data (
        steamid VARCHAR(30) PRIMARY KEY NOT NULL, 
        name VARCHAR(100), 
        kills INTEGER DEFAULT 0, 
        deaths INTEGER DEFAULT 0, 
        damage INTEGER DEFAULT 0, 
        tWins INTEGER DEFAULT 0, 
        ctWins INTEGER DEFAULT 0, 
        ctTotal INTEGER DEFAULT 0, 
        tTotal INTEGER DEFAULT 0, 
        bP INTEGER DEFAULT 0, 
        bD INTEGER DEFAULT 0, 
        hR INTEGER DEFAULT 0, 
        hK INTEGER DEFAULT 0, 
        bE INTEGER DEFAULT 0, 
        suicides INTEGER DEFAULT 0, 
        tP INTEGER DEFAULT 0, 
        hs INTEGER DEFAULT 0, 
        points INTEGER DEFAULT 0
        ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;""")
        
        self.cursor.execute("""\
        CREATE TABLE IF NOT EXISTS hitbox (
        steamid VARCHAR(30) PRIMARY KEY NOT NULL,
        head INTEGER DEFAULT 0, 
        chest INTEGER DEFAULT 0, 
        leftarm INTEGER DEFAULT 0, 
        rightarm INTEGER DEFAULT 0, 
        stomach INTEGER DEFAULT 0, 
        leftleg INTEGER DEFAULT 0, 
        rightleg INTEGER DEFAULT 0
        ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;""")
        
        #### MAP TABLE
        
        self.cursor.execute("""\
        CREATE TABLE IF NOT EXISTS maps (
        steamid VARCHAR(30) NOT NULL,
        map VARCHAR(150),
        time INTEGER DEFAULT 0,
        kills INTEGER DEFAULT 0,
        deaths INTEGER DEFAULT 0,
        headshots INTEGER DEFAULT 0,
        points INTEGER DEFAULT 0
        ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;""")

        string = "CREATE TABLE IF NOT EXISTS weapons (steamid VARCHAR(30) PRIMARY KEY NOT NULL"
        for weapon in rankme.all_weapons:
            string += ', %s INTEGER DEFAULT 0' % weapon
            if weapon.startswith('smoke'):
                string += ') ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;'
        self.cursor.execute(string)
        
    def execute(self, sql, args=None):
        ''' Executes certain actions in the database '''
        if args:
            self.cursor.execute(sql, args)
        else:
            self.cursor.execute(sql)
       
    def fetchone(self, sql):
        ''' Fetches data from the table from a single row '''
        self.cursor.execute(sql)
        return self.cursor.fetchone()
       
    def fetchall(self, sql):
        ''' Fetches data from the table from all rows '''
        self.cursor.execute(sql)
        return self.cursor.fetchall()
        
    def __delitem__(self, entry):
        self.execute('DELETE FROM data WHERE steamid="%s"' % entry)
        
    def delete(self):
        self.execute('TRUNCATE TABLE data')
        self.execute('TRUNCATE TABLE hitbox')
        self.execute('TRUNCATE TABLE weapons')
        self.execute('TRUNCATE TABLE maps')
        
mysql = MySQLDataBase()
        
def convertFromSQLite():
    """
    Convert the SQLite database into the MySQL database ensuring that all
    stats are made consistent throughout.
    """
    mysql.delete()
    
    datainsert   = "INSERT INTO data (steamid, name, kills, deaths, damage, tWins, ctWins, ctTotal, tTotal, bP, bD, hR, hK, bE, suicides, tP, hs, points) VALUES (\"%s\", \"%s\", %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
    hitboxinsert = "INSERT INTO hitbox (steamid, head, chest, leftarm, rightarm, stomach, leftleg, rightleg) VALUES (\"%s\", %s, %s, %s, %s, %s, %s, %s)"
    string = "INSERT INTO weapons (steamid"
    for weapon in rankme.all_weapons:
        string += ', %s' % weapon
        if weapon.startswith('smoke'):
            string += ')'
    
    weaponinsert = string + " VALUES (\"%s\", %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
    mapsinsert   = "INSERT INTO maps (steamid, map, time, kills, deaths, headshots, points) VALUES (\"%s\", \"%s\", %s, %s, %s, %s, %s)"
    players = rankme.database.getAllPlayers()
    if not players:
        return
    
    for player in players:
        if not rankme.database.getHitboxData(player):
            continue
        head, chest, stomach, leftarm, rightarm, leftleg, rightleg = rankme.database.getHitboxData(player)
        mysql.cursor.execute(hitboxinsert % (player, head, chest, stomach, leftarm, rightarm, leftleg, rightleg))
        
        if not rankme.database.getPlayer(player):
            continue
        kills, deaths, damage, tWins, ctWins, ctTotal, tTotal, bP, bD, hR, hK, bE, suicides, tP, hs, points = rankme.database.getPlayer(player)
        mysql.cursor.execute(datainsert % (player, re.escape(rankme.database.getPlayerName(player)), kills, deaths, damage, tWins, ctWins, ctTotal, tTotal, bP, bD, hR, hK, bE, suicides, tP, hs, points))
        
        if not rankme.database.getMapData(player):
            continue
        for entry in rankme.database.getMapData(player):
            time, kills, deaths, headshots, points = entry[1:]
        
            mysql.cursor.execute(mapsinsert % (player, entry[0], time, kills, deaths, headshots, points))
        
        if not rankme.database.getWeaponData(player):
            continue
        steamid, ak47, m4a1, deagle, glock, usp, p228, fiveseven, elite, m3, xm1014, tmp, mac10, ump45, mp5navy, p90, famas, galil, aug, sg552, scout, m249, awp, sg550, g3sg1, knife, hegrenade, flashbang, smokegrenade = rankme.database.getWeaponData(player)
        mysql.cursor.execute(weaponinsert % (player, ak47, m4a1, deagle, glock, usp, p228, fiveseven, elite, m3, xm1014, tmp, mac10, ump45, mp5navy, p90, famas, galil, aug, sg552, scout, m249, awp, sg550, g3sg1, knife, hegrenade, flashbang, smokegrenade))
    
    mysql.connection.commit()
    
def load():
    convertFromSQLite()
    
def es_map_start(ev):
    convertFromSQLite()